# coding: utf-8

from supervisely.api.entity_annotation.entity_annotation_api import EntityAnnotationAPI
from supervisely.api.module_api import ApiField
from supervisely.pointcloud_annotation.pointcloud_episode_annotation import PointcloudEpisodeAnnotation
from supervisely.video_annotation.key_id_map import KeyIdMap


class PointcloudEpisodeAnnotationAPI(EntityAnnotationAPI):
    _method_download = 'point-clouds.episodes.annotations.info'
    _entity_ids_str = ApiField.POINTCLOUD_IDS

    def download(self, dataset_id: int):
        """
        :param dataset_id: int
        :return: dataset episode annotation for given id, received after execution post request
        """
        response = self._api.post(self._method_download, {ApiField.DATASET_ID: dataset_id})
        return response.json()[0]

    def download_bulk(self, dataset_id, entity_ids):
        raise RuntimeError('Not supported for episodes')

    def append(self, dataset_id, ann: PointcloudEpisodeAnnotation, frame_to_pointcloud_ids,
               key_id_map: KeyIdMap = None) -> None:
        if key_id_map is None:
            # create for internal purposes (to link figures and tags to objects)
            key_id_map = KeyIdMap()

        figures = []
        pointcloud_ids = []
        for i, frame in enumerate(ann.frames):
            for fig in frame.figures:
                if frame_to_pointcloud_ids.get(i) is None:  # skip unmapped frames
                    continue

                figures.append(fig)
                pointcloud_ids.append(frame_to_pointcloud_ids[i])

        self._api.pointcloud_episode.object.append_to_dataset(dataset_id, ann.objects, key_id_map)
        self._api.pointcloud_episode.figure.append_to_dataset(dataset_id, figures, pointcloud_ids, key_id_map)




import os
from typing import List, Optional, Dict, Union, Tuple
from supervisely import Project, Api
from supervisely.project.project import ItemInfo
from supervisely.app import StateJson, DataJson
from supervisely.app.widgets import Widget, RadioTabs, Container, NotificationBox, SelectDataset, SelectString, Field, SelectTagMeta
from supervisely.app.widgets.random_splits_table.random_splits_table import RandomSplitsTable
from supervisely.app import get_data_dir
from supervisely._utils import rand_str
from supervisely.io.fs import remove_dir

class TrainValSplits(Widget):
    def __init__(
        self,
        project_id: Optional[int] = None,
        project_fs: Optional[Project] = None,
        random_splits: Optional[bool] = True,
        tags_splits: Optional[bool] = True,
        datasets_splits: Optional[bool] = True,
        widget_id: Optional[int] = None,
    ):
        self._project_id = project_id
        self._project_fs: Project = project_fs
        if project_fs is not None and project_id is not None:
            raise ValueError("You can not provide both project_id and project_fs parameters to TrainValSplits widget.")
        if project_fs is None and project_id is None:
            raise ValueError("You should provide at least one of: project_id or project_fs parameters to TrainValSplits widget.")
        
        self._project_info = None
        if project_id is not None:
            self._api = Api()
            self._project_info = self._api.project.get_info_by_id(self._project_id)
        self._random_splits_table: RandomSplitsTable = None
        self._train_tag_select: SelectTagMeta = None
        self._val_tag_select: SelectTagMeta = None
        self._untagged_select: SelectString = None
        self._train_ds_select: Union[SelectDataset, SelectString] = None
        self._val_ds_select: Union[SelectDataset, SelectString] = None
        self._split_methods = []
        contents = []
        tabs_descriptions = []
        if random_splits:
            self._split_methods.append('Random')
            tabs_descriptions.append('Shuffle data and split with defined probability')
            contents.append(self._get_random_content())
        if tags_splits:
            self._split_methods.append('Based on item tags')
            tabs_descriptions.append('Images should have assigned train or val tag')
            contents.append(self._get_tags_content())
        if datasets_splits:
            self._split_methods.append('Based on datasets')
            tabs_descriptions.append('Select one or several datasets for every split')
            contents.append(self._get_datasets_content())
        if not self._split_methods:
            raise ValueError("Any of split methods [random_splits, tags_splits, datasets_splits] must be specified in TrainValSplits.")
        
        self._content = RadioTabs(
            titles=self._split_methods,
            descriptions=tabs_descriptions,
            contents=contents,
        )

        super().__init__(widget_id=widget_id, file_path=__file__)
    
    def _get_random_content(self):
        if self._project_id is not None:
            items_count = self._project_info.items_count
        elif self._project_fs is not None:
            items_count = self._project_fs.total_items
        self._random_splits_table = RandomSplitsTable(items_count)
        
        return Container(
            widgets=[self._random_splits_table],
            direction='vertical',
            gap = 5
        )
    
    def _get_tags_content(self):
        notification_box = NotificationBox(
            title="Notice: How to make equal splits",
            description="Choose the same tag for train/validation to make splits equal. Can be used for debug and for tiny projects",
            box_type='info'
        )
        if self._project_id is not None:
            self._train_tag_select = SelectTagMeta(project_id=self._project_id, show_label=False)
            self._val_tag_select = SelectTagMeta(project_id=self._project_id, show_label=False)
        elif self._project_fs is not None:
            self._train_tag_select = SelectTagMeta(project_meta=self._project_fs.meta, show_label=False)
            self._val_tag_select = SelectTagMeta(project_meta=self._project_fs.meta, show_label=False)
        self._untagged_select = SelectString(
            values = ["train", "val", "ignore"],
            labels=[
                "add untagged images to train set",
                "add untagged images to val set",
                "ignore untagged images", 
            ],
            placeholder="Select action"
        )
        train_field = Field(
            self._train_tag_select, 
            title="Train tag",
            description="all images with this tag are considered as training set"
        )
        val_field = Field(
            self._val_tag_select,
            title="Validation tag",
            description="all images with this tag are considered as validation set"
        )
        without_tags_field = Field(
            self._untagged_select,
            title="Images without selected tags",
            description="Choose what to do with untagged images"
        )
        return Container(
            widgets=[
                notification_box,
                train_field,
                val_field,
                without_tags_field,
            ],
            direction='vertical',
            gap = 5
        )
    
    def _get_datasets_content(self):
        notification_box = NotificationBox(
            title="Notice: How to make equal splits",
            description="Choose the same dataset(s) for train/validation to make splits equal. Can be used for debug and for tiny projects",
            box_type='info'
        )
        if self._project_id is not None:
            self._train_ds_select = SelectDataset(
                project_id=self._project_id, 
                multiselect=True, 
                compact=True, 
                show_label=False
            )
            self._val_ds_select = SelectDataset(
                project_id=self._project_id, 
                multiselect=True, 
                compact=True, 
                show_label=False
            )
        elif self._project_fs is not None:
            ds_names = [ds.name for ds in self._project_fs.datasets]
            self._train_ds_select = SelectString(ds_names, multiple=True)
            self._val_ds_select = SelectString(ds_names, multiple=True)
        train_field = Field(
            self._train_ds_select, 
            title="Train dataset(s)",
            description="all images in selected dataset(s) are considered as training set"
        )
        val_field = Field(
            self._val_ds_select,
            title="Validation dataset(s)",
            description="all images in selected dataset(s) are considered as validation set"
        )
        return Container(
            widgets=[notification_box, train_field, val_field],
            direction='vertical',
            gap = 5)

    def get_json_data(self):
        return {}

    def get_json_state(self):
        return {}

    def get_splits(self) -> Tuple[List[ItemInfo], List[ItemInfo]]:
        split_method = self._content.get_active_tab()
        tmp_project_dir = None
        if self._project_fs is None:
            tmp_project_dir = os.path.join(get_data_dir(), rand_str(15))
            Project.download(self._api, self._project_id, tmp_project_dir)
        project_dir = tmp_project_dir if tmp_project_dir is not None else self._project_fs.directory
        if split_method == 'Random':
            splits_counts = self._random_splits_table.get_splits_counts()
            train_count = splits_counts["train"]
            val_count = splits_counts["val"]
            train_set, val_set = Project.get_train_val_splits_by_count(project_dir, train_count, val_count)
            
        elif split_method == 'Based on item tags':
            train_tag_name = self._train_tag_select.get_selected_name()
            val_tag_name = self._val_tag_select.get_selected_name()
            add_untagged_to = self._untagged_select.get_value()
            train_set, val_set = Project.get_train_val_splits_by_tag(project_dir, train_tag_name, val_tag_name, add_untagged_to)
            
        elif split_method == 'Based on datasets':
            if self._project_id is not None:
                self._train_ds_select: SelectDataset
                self._val_ds_select: SelectDataset
                train_ds_ids = self._train_ds_select.get_selected_ids()
                val_ds_ids = self._val_ds_select.get_selected_ids()
                ds_infos = self._api.dataset.get_list(self._project_id)
                train_ds_names, val_ds_names = [], []
                for ds_info in ds_infos:
                    if ds_info.id in train_ds_ids:
                        train_ds_names.append(ds_info.name)
                    if ds_info.id in val_ds_ids:
                        val_ds_names.append(ds_info.name)
            elif self._project_fs is not None:
                self._train_ds_select: SelectString
                self._val_ds_select: SelectString
                train_ds_names = self._train_ds_select.get_value()
                val_ds_names = self._val_ds_select.get_value()
            train_set, val_set = Project.get_train_val_splits_by_dataset(project_dir, train_ds_names, val_ds_names)
            
        if tmp_project_dir is not None:
            remove_dir(tmp_project_dir)
        return train_set, val_set

    def disable(self):
        self._content.disable()
        self._random_splits_table.disable()
        self._train_tag_select.disable()
        self._val_tag_select.disable()
        self._untagged_select.disable()
        self._train_ds_select.disable()
        self._val_ds_select.disable()
        self._disabled = True
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()

    def enable(self):
        self._content.enable()
        self._random_splits_table.enable()
        self._train_tag_select.enable()
        self._val_tag_select.enable()
        self._untagged_select.enable()
        self._train_ds_select.enable()
        self._val_ds_select.enable()
        self._disabled = False
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()


import os
import json
import random
from typing import List, Optional, Dict, Literal
from collections import namedtuple

import supervisely as sly
from supervisely.app import StateJson, DataJson
from supervisely.app.widgets import Container, Widget, RadioTabs, Editor, GridGallery, Select, Input, Field, Button


class Augmentations(Widget):
    def __init__(
        self,
        globals,
        image_info: sly.ImageInfo = None, 
        task_type: Optional[Literal['detection', 'segmentation']] = None,
        remote_preview_path: str = '/temp/preview_augs.jpg',
        widget_id=None
    ):  
        self._globals = globals
        self._image_info = image_info
        self._task_type = task_type
        self._remote_preview_path = remote_preview_path

        self._template_path  = None
        self._pipeline = None
        self._py_code = None

        self._editor = Editor(
            language_mode='python', 
            height_px=250, 
            readonly=True, 
            show_line_numbers=True,
            highlight_active_line=False,
            restore_default_button=False,
        )
        self._grid_gallery = GridGallery(
            columns_number=2,
            annotations_opacity=0.3,
            show_opacity_slider=True,
            enable_zoom=True,
            resize_on_zoom=False,
            sync_views=True,
            fill_rectangle=False,
        )
        self._grid_gallery.hide()
        self._button_preview = Button('Preview on random image')
        self._content = Container([self._editor, self._grid_gallery, self._button_preview])
        
        @self._button_preview.click
        def update_preview():
            self.preview_augs()
        super().__init__(widget_id=widget_id, file_path=__file__)
    
    def preview_augs(self, image_info: sly.ImageInfo = None):
        if not image_info:
            ds_name, item_name = self.get_random_item()
            self._image_info = self.get_image_info_from_cache(ds_name, item_name)

        img = self._globals.api.image.download_np(self._image_info.id)
        ann_json = self._globals.api.annotation.download(self._image_info.id).annotation

        image_ann = sly.Annotation.from_json(ann_json, self._globals.project_meta)
        meta = self._globals.project_meta
        if self._task_type == "detection":
            image_ann, meta = self.convert_ann_to_bboxes(image_ann, self._globals.project_meta)

        _, res_img, res_ann = sly.imgaug_utils.apply(self._pipeline, meta, img, image_ann)
        local_image_path = os.path.join(self._globals.data_dir, "preview_augs.jpg")
        sly.image.write(local_image_path, res_img)
        if self._globals.api.file.exists(self._globals.team.id, self._remote_preview_path):
            self._globals.api.file.remove(self._globals.team.id, self._remote_preview_path)
        file_info = self._globals.api.file.upload(self._globals.team.id, local_image_path, self._remote_preview_path)
        
        self._grid_gallery.clean_up()
        self._grid_gallery.append(
            title=f"Original", image_url=self._image_info.full_storage_url, annotation=image_ann
        )
        self._grid_gallery.append(
            title=f"Augmented", image_url=file_info.full_storage_url, annotation=res_ann
        )
        self._grid_gallery.show()
    
    def get_random_item(self):
        all_ds_names = self._globals.project_fs.datasets.keys()
        ds_name = random.choice(all_ds_names)
        ds = self._globals.project_fs.datasets.get(ds_name)
        items = list(ds)
        item_name = random.choice(items)
        return ds_name, item_name
    
    def get_image_info_from_cache(self, dataset_name, item_name):
        dataset_fs = self._globals.project_fs.datasets.get(dataset_name)
        img_info_path = dataset_fs.get_img_info_path(item_name)
        image_info_dict = sly.json.load_json_file(img_info_path)
        ImageInfo = namedtuple('ImageInfo', image_info_dict)
        info = ImageInfo(**image_info_dict)
        return info

    def update_augmentations(self, path_or_data: str = None, string_format: str = 'python'):
        if path_or_data.endswith('.json'):
            self._template_path = path_or_data
            config = sly.json.load_json_file(self._template_path)
            self._pipeline, self._py_code = self.load_augs_template(config)
        elif path_or_data.endswith('.py'):
            # TODO add support .py files
            raise NotImplementedError('.py files not supported yet.')
        else:
            if string_format == 'json':
                config = json.loads(path_or_data)
                self._pipeline, self._py_code = self.load_augs_template(config)
            elif string_format == 'python':
                # TODO create func to conversion augs  
                raise NotImplementedError('Raw python augmentations not supported yet.')
            else:
                raise ValueError('Supported values for "string_format" is "python" or "json"')
        self._editor.set_text(text=self._py_code)
    
    def get_augmentations(self):
        return self._pipeline, self._py_code

    def get_json_data(self) -> Dict:
        return {}

    def get_json_state(self) -> Dict:
        return {}
    
    @staticmethod
    def convert_ann_to_bboxes(image_ann, project_meta):
        meta = project_meta.clone()
        for obj_class in meta.obj_classes:
            if obj_class.geometry_type == "rectangle":
                continue
            class_obj = sly.ObjClass(obj_class.name, sly.Rectangle, obj_class.color)
            meta = meta.delete_obj_class(obj_class.name)
            meta = meta.add_obj_class(class_obj)
        new_ann_json = {
            "size": {
                "height": image_ann.img_size[0],
                "width": image_ann.img_size[1]
            },
            "tags": [],
            "objects": []
        }
        new_ann = sly.Annotation.from_json(new_ann_json, meta)
        for label in image_ann.labels:
            if label.geometry.geometry_name == "rectangle":
                new_ann = image_ann.add_label(label)
                continue
            class_obj = sly.ObjClass(label.obj_class.name, sly.Rectangle, label.obj_class.color)
            updated_label = label.convert(class_obj)[0]
            new_ann = new_ann.add_label(updated_label)
        return new_ann, meta

    @staticmethod
    def load_augs_template(config: dict):
        pipeline = sly.imgaug_utils.build_pipeline(config["pipeline"], random_order=config["random_order"]) # to validate
        py_code = sly.imgaug_utils.pipeline_to_python(config["pipeline"], config["random_order"])
        return pipeline, py_code

    def disable(self):
        self._disabled = True
        self._editor.disable()
        self._grid_gallery.disable()
        self._button_preview.disable()
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()

    def enable(self):
        self._disabled = False
        self._editor.enable()
        self._grid_gallery.enable()
        self._button_preview.enable()
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()


class AugmentationsWithTabs(Widget):
    def __init__(
        self,
        globals,
        templates: List[Dict[str, str]],
        image_info: sly.ImageInfo = None, 
        task_type: Optional[Literal['detection', 'segmentation']] = None,
        remote_preview_path: str = '/temp/preview_augs.jpg',
        widget_id=None
    ):  
        self._globals = globals
        self._image_info = image_info
        self._templates = templates
        self._task_type = task_type
        self._remote_preview_path = remote_preview_path
        
        self._augs1 = Augmentations(globals, image_info=self._image_info, task_type=self._task_type)
        self._augs2 = Augmentations(globals, image_info=self._image_info, task_type=self._task_type)
        self._augs2._editor.hide()
        self._augs2._button_preview.hide()
        self._template_path_input = Input(placeholder="Path to .json file in Team Files")
        self._template_selector = Select(
            items=[Select.Item(value=t['value'], label=t['label']) for t in templates], 
            filterable=True, 
            placeholder="select me"
        )
        self._button_template_update = Button('Load template from file')
        self._radio_tabs = RadioTabs(
            titles=["From template", "Custom pipeline"],
            contents=[
                Field(
                    title='Template', 
                    content=Container([self._template_selector, self._augs1]),
                ),
                Field(
                    title='Path to JSON configuration', 
                    description='Copy path in Team Files', 
                    content=Container([self._template_path_input, self._button_template_update, self._augs2])
                )
            ],
            descriptions=[
                "Choose one of the prepared templates",
                "Use ImgAug Studio appto configure and save custom augmentations",
            ],
        )
        
        self._current_augs = self._augs1
        self._current_augs.update_augmentations(templates[0]['value'])

        @self._radio_tabs.value_changed
        def tab_toggle(tab_title):
            if tab_title == 'Custom pipeline':
                self._current_augs = self._augs2
            else:
                self._current_augs = self._augs1

        @self._template_selector.value_changed
        def selector_value_changed(value = None):
            self._current_augs.update_augmentations(value)

        @self._button_template_update.click
        def update_template():
            custom_template_path = self._template_path_input.get_value()
            if self._globals.api.file.exists(team_id=self._globals.team.id, remote_path=custom_template_path.strip()):
                self.load_existing_pipeline(custom_template_path)
                self._augs2._editor.show()
                self._augs2._button_preview.show()
            else:
                raise FileExistsError("Team files does't contains file by passed path.")

        self._content = Container([self._radio_tabs])
        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self) -> Dict:
        return {}

    def get_json_state(self) -> Dict:
        return {}
  
    def get_augmentations(self):
        return self._current_augs.get_augmentations()
    
    def load_existing_pipeline(self, remote_path):
        custom_pipeline_path = os.path.join(self._globals.data_dir, sly.fs.get_file_name_with_ext(remote_path))
        self._globals.api.file.download(self._globals.team.id, remote_path, custom_pipeline_path)
        self._current_augs.update_augmentations(custom_pipeline_path)

    def disable(self):
        self._disabled = True
        self._radio_tabs.disable()
        self._current_augs.disable()
        self._template_selector.disable()
        self._template_path_input.disable()
        self._button_template_update.disable()
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()

    def enable(self):
        self._disabled = False
        self._radio_tabs.enable()
        self._current_augs.enable()
        self._template_selector.enable()
        self._template_path_input.enable()
        self._button_template_update.enable()
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()


from __future__ import annotations
from typing import Optional, List, Dict, Any, Iterator

from supervisely.video_annotation.key_id_map import KeyIdMap
from supervisely.annotation.tag_collection import TagCollection
from supervisely.annotation.tag_meta_collection import TagMetaCollection
from supervisely.pointcloud_annotation.pointcloud_tag import PointcloudTag
from supervisely.annotation.tag_meta import TagMeta
import supervisely.sly_logger as logger


class PointcloudTagCollection(TagCollection):
    """
    Collection with :class:`PointcloudTag<supervisely.pointcloud_annotation.pointcloud_tag.PointcloudTag>` instances.
    :class:`PointcloudTagCollection<PointcloudTagCollection>` object is immutable.

    :Usage example:

     .. code-block:: python

        import supervisely as sly

        # Create TagMetas (see class TagMeta for additional information about creating TagMetas)
        meta_weather = sly.TagMeta('Weather', sly.TagValueType.ANY_STRING)

        seasons = ["Winter", "Spring", "Summer", "Autumn"]
        meta_season = sly.TagMeta('Season', sly.TagValueType.ONEOF_STRING, possible_values=seasons)

        # Create Pointcloud Tags
        tag_weather = sly.PointcloudTag(meta_weather, value="Sunny")
        tag_season = sly.PointcloudTag(meta_season, value="Spring")
        tags_arr = [tag_weather, tag_season]

        # Create PointcloudTagCollection from Pointcloud Tags
        tags = sly.PointcloudTagCollection(tags_arr)

        # Add item to PointcloudTagCollection
        meta_potato = sly.TagMeta('potato', sly.TagValueType.NONE)
        tag_potato = sly.PointcloudTag(meta_potato)

        # Remember that PointcloudTagCollection is immutable, and we need to assign new instance of PointcloudTagCollection to a new variable
        tags = tags.add(tag_potato)

        # You can also add multiple items to collection
        meta_cabbage = sly.TagMeta('cabbage', sly.TagValueType.NONE)
        meta_carrot = sly.TagMeta('carrot', sly.TagValueType.NONE)
        meta_turnip = sly.TagMeta('turnip', sly.TagValueType.NONE)

        tag_cabbage = sly.PointcloudTag(meta_cabbage)
        tag_carrot = sly.PointcloudTag(meta_carrot)
        tag_turnip = sly.PointcloudTag(meta_turnip)

        additional_veggies = [tag_cabbage, tag_carrot, tag_turnip]

        tags = tags.add_items(additional_veggies)

        # Has key, checks if given key exist in collection
        tags.has_key(tag_cabbage.key())
        # Output: True

        # Intersection, finds intersection of given list of instances with collection items
        meta_dog = sly.TagMeta('dog', sly.TagValueType.NONE)
        tag_dog = sly.PointcloudTag(meta_dog)

        meta_cat = sly.TagMeta('cat', sly.TagValueType.NONE)
        tag_cat = sly.PointcloudTag(meta_cat)

        meta_turtle = sly.TagMeta('turtle', sly.TagValueType.NONE)
        tag_turtle = sly.PointcloudTag(meta_turtle)


        tags_animals = sly.PointcloudTagCollection([tag_dog, tag_cat, tag_turtle])

        tags_intersections = tags.intersection(tags_animals)
        print(tags_intersections.to_json())
        # Output: []

        # Let's add the potato Tag from another collection and compare them again
        tags_animals = tags_animals.add(tag_potato)

        tags_intersections = tags.intersection(tags_animals)
        print(tags_intersections.to_json())
        # Output: [
        #     {
        #         "name":"potato",
        #         "key": "058ad7993a534082b4d94cc52542a97d"
        #     }
        # ]

        # Difference, finds difference between collection and given list of Tags or TagCollection
        meta_car = sly.TagMeta('car', sly.TagValueType.NONE)
        tag_car = sly.Tag(meta_car)

        meta_bicycle = sly.TagMeta('bicycle', sly.TagValueType.NONE)
        tag_bicycle = sly.PointcloudTag(meta_bicycle)

        tags_vehicles = sly.PointcloudTagCollection([tag_car, tag_bicycle])

        meta_pedestrian = sly.TagMeta('pedestrian', sly.TagValueType.NONE)
        tag_pedestrian = sly.PointcloudTag(meta_pedestrian)

        meta_road = sly.TagMeta('road', sly.TagValueType.NONE)
        tag_road = sly.PointcloudTag(meta_road)

        difference = tags_vehicles.difference([tag_pedestrian, tag_road])
        print(difference.to_json())
        # Output: [
        #     {
        #         "name": "car",
        #         "key": "058ad7993a534082b4d94cc52542a97d"
        #     },
        #     {
        #         "name": "bicycle",
        #         "key": ""0c0033c5b4834d4cbabece4317295f07"
        #     }
        # ]

        # Merge, merges collection and given list of TagMetas
        tags_vehicles = sly.TagMetaCollection([tag_car, tag_bicycle])
        tags_merge = sly.TagMetaCollection([tag_pedestrian, tag_road])

        merged_collections = tags_vehicles.merge(tags_merge)
        print(merged_collections.to_json())
        # Output: [
        #     {
        #         "name":"pedestrian"
        #     },
        #     {
        #         "name":"road"
        #     },
        #     {
        #         "name":"car"
        #     },
        #     {
        #         "name":"bicycle"
        #     }
        # ]
    """

    item_type = PointcloudTag

    def __iter__(self) -> Iterator[PointcloudTag]:
        return next(self)

    # @classmethod
    # def from_api_response(
    #     cls,
    #     data: List[Dict],
    #     tag_meta_collection: TagMetaCollection,
    #     id_to_tagmeta: Optional[Dict[int, TagMeta]] = None
    # ) -> PointcloudTagCollection:
    #     return super().from_api_response(data, tag_meta_collection, id_to_tagmeta)

    # def get_by_name(self, tag_name: str) -> List[PointcloudTag]:
    #     """
    #     Get list of Pointcloud Tags with provided name.

    #     :param tag_name: Pointcloud Tag name.
    #     :type tag_name: :class:`str`
    #     :return: List of Pointcloud Tags.
    #     :rtype: :class:`List[PointcloudTag]<supervisely.pointcloud_annotation.pointcloud_tag.PointcloudTag>`
    #     """
    #     res = []
    #     for tag in self:
    #         if tag.name == tag_name:
    #             res.append(tag)
    #     return res

    # def get_single_by_name(self, tag_name: str) -> PointcloudTag:
    #     """
    #     Get Pointcloud Tag with provided name.
    #     Method will raise error If collection contains more than 1 tag with provided name.

    #     :param tag_name: Pointcloud Tag name.
    #     :type tag_name: :class:`str`
    #     :return: PointcloudTag object or :class:`None<None>` If no elements with provided name in collection.
    #     :rtype: :class:`PointcloudTag<supervisely.pointcloud_annotation.pointcloud_tag.PointcloudTag>` or :class:`NoneType<NoneType>`
    #     :raises: :class:`ValueError`, If collection contains more than 1 tag with provided name.
    #     """
    #     res = []
    #     for tag in self:
    #         if tag.name == tag_name:
    #             res.append(tag)
    #     if len(res) == 0:
    #         return None
    #     if len(res) > 1:
    #         raise ValueError(
    #             f"There are more than one tag {tag_name} in VideoTagCollection. Use method get_by_name instead"
    #         )
    #     return res[0]

    def to_json(self, key_id_map: Optional[KeyIdMap] = None) -> List[Dict]:
        """
        Convert the :class:`PointcloudTagCollection<PointcloudTagCollection>` to a list of json dicts.
        Read more about `Supervisely format <https://docs.supervise.ly/data-organization/00_ann_format_navi>`_.

        :param key_id_map: KeyIdMap object.
        :type key_id_map: :class:`KeyIdMap<supervisely.video_annotation.key_id_map.KeyIdMap>`, optional
        :return: List of dicts in json format
        :rtype: :class:`List[dict]`
        :Usage example:

         .. code-block:: python

            import supervisely as sly

            meta_weather = sly.TagMeta('Weather', sly.TagValueType.ANY_STRING)

            seasons = ["Winter", "Spring", "Summer", "Autumn"]
            meta_season = sly.TagMeta('Season', sly.TagValueType.ONEOF_STRING, possible_values=seasons)

            tag_weather = sly.PointcloudTag(meta_weather, value="Sunny")
            tag_season = sly.PointcloudTag(meta_season, value="Spring")

            tags_arr = [tag_weather, tag_season]
            tags = sly.PointcloudTagCollection(tags_arr)

            tags_json = tags.to_json()
            # Output: [
            #     {
            #         "name":"Weather",
            #         "value":"Sunny",
            #         "key": "058ad7993a534082b4d94cc52542a97d"
            #     },
            #     {
            #         "name":"Season",
            #         "value":"Spring",
            #         "key": "0c0033c5b4834d4cbabece4317295f07"
            #     }
            # ]
        """
        return [tag.to_json(key_id_map) for tag in self]

    @classmethod
    def from_json(
        cls,
        data: List[Dict],
        tag_meta_collection: TagMetaCollection,
        key_id_map: Optional[KeyIdMap] = None,
    ) -> PointcloudTagCollection:
        """
        Convert a list with dicts in json format to :class:`PointcloudTagCollection<PointcloudTagCollection>`.
        Read more about `Supervisely format <https://docs.supervise.ly/data-organization/00_ann_format_navi>`.

        :param data: List with dicts in json format.
        :type data: :class:`List[Dict]`
        :param tag_meta_collection: TagMetaCollection object.
        :type tag_meta_collection: :class:`TagMetaCollection<supervisely.annotation.tag_meta_collection.TagMetaCollection>`
        :param key_id_map: KeyIdMap object.
        :type key_id_map: :class:`KeyIdMap<supervisely.video_annotation.key_id_map.KeyIdMap>`, optional
        :return: TagCollection object.
        :rtype: :class:`PointcloudTagCollection<PointcloudTagCollection>`
        :Usage example:

         .. code-block:: python

            import supervisely as sly

            # Initialize TagMetaCollection

            meta_weather = sly.TagMeta('Weather', sly.TagValueType.ANY_STRING)

            seasons = ["Winter", "Spring", "Summer", "Autumn"]
            meta_season = sly.TagMeta('Season', sly.TagValueType.ONEOF_STRING, possible_values=seasons)

            metas_arr = [meta_weather, meta_season]
            tag_metas = sly.TagMetaCollection(metas_arr)

            data = [
                {
                    "name":"Weather",
                    "value":"Sunny"
                },
                {
                    "name":"Season",
                    "value":"Spring"
                }
            ]

            tags = sly.PointcloudTagCollection.from_json(data, tag_metas)
        """
        tags = [
            cls.item_type.from_json(tag_json, tag_meta_collection, key_id_map) for tag_json in data
        ]
        return cls(tags)



# coding: utf-8

# docs
from typing import Tuple, Optional

import os

from PIL import ImageFont
from supervisely.io.fs import get_file_ext, file_exists

FONT_EXTENSION = ".ttf"
DEFAULT_FONT_FILE_NAME = "DejaVuSansMono.ttf"
_fonts = {}  # (font name, size) -> font


def _get_font_path_by_name(font_file_name: str) -> str:
    """
    Walk over systems fonts paths and match with given font file name.

    :param font_file_name: Nameame of the font file.
    :type font_file_name: str
    :return: Full path of requested by name font or None if file not found in system paths.
    :rtype: str
    """
    import matplotlib.font_manager as fontman

    fonts_paths_list = fontman.findSystemFonts()
    for path in fonts_paths_list:
        if os.path.basename(path) == font_file_name:
            return path
    return None


def load_font(
        font_file_name: str, font_size: Optional[int] = 12
) -> ImageFont.FreeTypeFont:
    """
    Set global font true-type for drawing.

    :param font_file_name: name of font file (example: 'DejaVuSansMono.ttf')
    :type font_file_name: str
    :param font_size: selected font size
    :type font_size: int
    :return: Font object
    :rtype: PIL.ImageFont.FreeTypeFont
    """
    if get_file_ext(font_file_name) == FONT_EXTENSION:
        font_path = _get_font_path_by_name(font_file_name)
        if (font_path is not None) and file_exists(font_path):
            return ImageFont.truetype(font_path, font_size, encoding="utf-8")
        else:
            raise ValueError(
                'Font file "{}" not found in system paths. Try to set another font.'.format(
                    font_file_name
                )
            )
    else:
        raise ValueError("Supported only TrueType fonts!")


def get_font(
        font_file_name: Optional[str] = None, font_size: Optional[int] = 12
) -> ImageFont.FreeTypeFont:
    """
    Args:
        font_file_name: name of font file (example: 'DejaVuSansMono.ttf')
        font_size: selected font size
    Returns:
        font for drawing
    """
    if font_file_name is None:
        font_file_name = DEFAULT_FONT_FILE_NAME

    font_key = (font_file_name, font_size)
    if font_key not in _fonts:
        _fonts[font_key] = load_font(font_file_name, font_size)
    return _fonts[font_key]


def get_readable_font_size(img_size: Tuple[int, int]) -> int:
    """
    Get size of font for image with given sizes
    :param img_size: size of image
    :return: size of font
    """
    minimal_font_size = 6
    base_font_size = 14
    base_image_size = 512
    return max(
        minimal_font_size,
        round(base_font_size * (img_size[0] + img_size[1]) // 2) // base_image_size,
    )



# coding: utf-8
from __future__ import annotations

import os
import json
from typing import NamedTuple, List, Dict, Optional
from supervisely.api.module_api import ApiField
from supervisely.api.task_api import TaskApi
from supervisely._utils import take_with_default

# from supervisely.app.constants import DATA, STATE, CONTEXT, TEMPLATE
STATE = "state"
DATA = "data"
TEMPLATE = "template"

from supervisely.io.fs import ensure_base_path
from supervisely.task.progress import Progress
from supervisely._utils import sizeof_fmt
from supervisely import logger

_context_menu_targets = {
    "files_folder": {
        "help": "Context menu of folder in Team Files. Target value is directory path.",
        "type": str,
        "key": "slyFolder",
    },
    "files_file": {
        "help": "Context menu of file in Team Files. Target value is file path.",
        "type": str,
        "key": "slyFile",
    },
    "images_project": {
        "help": "Context menu of images project. Target value is project id.",
        "type": int,
        "key": "slyProjectId",
    },
    "images_dataset": {
        "help": "Context menu of images dataset. Target value is dataset id.",
        "type": int,
        "key": "slyDatasetId",
    },
    "videos_project": {
        "help": "Context menu of videos project. Target value is project id.",
        "type": int,
        "key": "slyProjectId",
    },
    "videos_dataset": {
        "help": "Context menu of videos dataset. Target value is dataset id.",
        "type": int,
        "key": "slyDatasetId",
    },
    "point_cloud_episodes_project": {
        "help": "Context menu of pointcloud episodes project. Target value is project id.",
        "type": int,
        "key": "slyProjectId",
    },
    "point_cloud_episodes_dataset": {
        "help": "Context menu of pointcloud episodes dataset. Target value is dataset id.",
        "type": int,
        "key": "slyDatasetId",
    },
    "point_cloud_project": {
        "help": "Context menu of pointclouds project. Target value is project id.",
        "type": int,
        "key": "slyProjectId",
    },
    "point_cloud_dataset": {
        "help": "Context menu of pointclouds dataset. Target value is dataset id.",
        "type": int,
        "key": "slyDatasetId",
    },
    "volumes_project": {
        "help": "Context menu of volumes project (DICOMs). Target value is project id.",
        "type": int,
        "key": "slyProjectId",
    },
    "volumes_dataset": {
        "help": "Context menu of volumes dataset (DICOMs). Target value is dataset id.",
        "type": int,
        "key": "slyDatasetId",
    },
    "team": {
        "help": "Context menu of team. Target value is team id.",
        "type": int,
        "key": "slyTeamId",
    },
    "team_member": {
        "help": "Context menu of team member. Target value is user id.",
        "type": int,
        "key": "slyMemberId",
    },
    "labeling_job": {
        "help": "Context menu of labeling job. Target value is labeling job id.",
        "type": int,
        "key": "slyJobId",
    },
    "ecosystem": {
        "help": "Run button in ecosystem. It is not needed to define any target",
        "key": "nothing",
    },
}


class AppInfo(NamedTuple):
    """AppInfo"""

    id: int
    created_by_id: int
    module_id: int
    disabled: bool
    user_login: str
    config: dict
    name: str
    slug: str
    is_shared: bool
    tasks: List[Dict]
    repo: str
    team_id: int


class ModuleInfo(NamedTuple):
    """ModuleInfo in Ecosystem"""

    id: int
    slug: str
    name: str
    type: str
    config: dict
    readme: str
    repo: str
    github_repo: str
    meta: dict
    created_at: str
    updated_at: str

    @staticmethod
    def from_json(data: dict) -> ModuleInfo:
        info = ModuleInfo(
            id=data["id"],
            slug=data["slug"],
            name=data["name"],
            type=data["type"],
            config=data["config"],
            readme=data.get("readme"),
            repo=data.get("repositoryModuleUrl"),
            github_repo=data.get("repo"),
            meta=data.get("meta"),
            created_at=data["createdAt"],
            updated_at=data["updatedAt"],
        )
        if "contextMenu" in info.config:
            info.config["context_menu"] = info.config["contextMenu"]
        return info

    def get_latest_release(self, default=""):
        release = self.meta.get("releases", [default])[0]
        return release

    def arguments_help(self):
        modal_args = self.get_modal_window_arguments()
        if len(modal_args) == 0:
            print(
                f"App '{self.name}' has no additional options \n"
                "that can be configured manually in modal dialog window \n"
                "before running app."
            )
        else:
            print(
                f"App '{self.name}' has additional options "
                "that can be configured manually in modal dialog window before running app. "
                "You can change them or keep defaults: "
            )
            print(json.dumps(modal_args, sort_keys=True, indent=4))

        targets = self.get_context_menu_targets()

        if len(targets) > 0:
            print("App has to be started from the context menus:")
            for target in targets:
                print(
                    f'{target} : {_context_menu_targets.get(target, {"help": "empty description"})["help"]}'
                )
            print(
                "It is needed to call get_arguments method with defined target argument (pass one of the values above)."
            )

        if "ecosystem" in targets:
            pass

    def get_modal_window_arguments(self):
        params = self.config.get("modalTemplateState", {})
        return params

    def get_arguments(self, **kwargs) -> dict:
        params = self.config.get("modalTemplateState", {})
        targets = self.get_context_menu_targets()
        if len(targets) > 0 and len(kwargs) == 0 and "ecosystem" not in targets:
            raise ValueError(
                "target argument has to be defined. Call method 'arguments_help' to print help info for developer"
            )
        if len(kwargs) > 1:
            raise KeyError("Only one target is allowed")
        if len(kwargs) == 1:
            # params["state"] = {}
            for target_key, target_value in kwargs.items():
                if target_key not in targets:
                    raise KeyError(
                        f"You passed {target_key}, but allowed only one of the targets: {targets}"
                    )
                key = _context_menu_targets[target_key]["key"]
                valid_type = _context_menu_targets[target_key]["type"]
                if type(target_value) is not valid_type:
                    raise ValueError(
                        f"Target {target_key} has value {target_value} of type {type(target_value)}. Allowed type is {valid_type}"
                    )
                params[key] = target_value
        return params

    def get_context_menu_targets(self):
        if "context_menu" in self.config:
            if "target" in self.config["context_menu"]:
                return self.config["context_menu"]["target"]
        return []


class SessionInfo(NamedTuple):
    """SessionInfo"""

    task_id: int
    user_id: int
    module_id: int  # in ecosystem
    app_id: int  # in team (recent apps)

    details: dict

    @staticmethod
    def from_json(data: dict) -> SessionInfo:
        # {'taskId': 21012, 'userId': 6, 'moduleId': 83, 'appId': 578}

        if "meta" in data:
            info = SessionInfo(
                task_id=data["id"],
                user_id=data["createdBy"],
                module_id=data["moduleId"],
                app_id=data["meta"]["app"]["id"],
                details=data,
            )
        else:
            info = SessionInfo(
                task_id=data["taskId"],
                user_id=data["userId"],
                module_id=data["moduleId"],
                app_id=data["appId"],
                details={},
            )
        return info


class AppApi(TaskApi):
    """AppApi"""

    @staticmethod
    def info_sequence():
        """info_sequence"""
        return [
            ApiField.ID,
            ApiField.CREATED_BY_ID,
            ApiField.MODULE_ID,
            ApiField.DISABLED,
            ApiField.USER_LOGIN,
            ApiField.CONFIG,
            ApiField.NAME,
            ApiField.SLUG,
            ApiField.IS_SHARED,
            ApiField.TASKS,
            ApiField.REPO,
            ApiField.TEAM_ID,
        ]

    @staticmethod
    def info_tuple_name():
        """info_tuple_name"""
        return "AppInfo"

    def _convert_json_info(self, info: dict, skip_missing=True) -> AppInfo:
        """_convert_json_info"""
        res = super(TaskApi, self)._convert_json_info(info, skip_missing=skip_missing)
        return AppInfo(**res._asdict())

    def get_info_by_id(self, id: int) -> AppInfo:
        """
        :param id: int
        :return: application info by numeric id
        """
        return self._get_info_by_id(id, "apps.info")

    def get_list(
        self,
        team_id,
        filter=None,
        context=None,
        repository_key=None,
        show_disabled=False,
        integrated_into=None,
        session_tags=None,
        only_running=False,
        with_shared=True,
    ) -> List[AppInfo]:
        """get_list"""

        return self.get_list_all_pages(
            method="apps.list",
            data={
                "teamId": team_id,
                "filter": take_with_default(
                    filter, []
                ),  # for example [{"field": "id", "operator": "=", "value": None}]
                "context": take_with_default(context, []),  # for example ["images_project"]
                "repositoryKey": repository_key,
                "integratedInto": take_with_default(
                    integrated_into, []
                ),  # for example ["image_annotation_tool"]
                "sessionTags": take_with_default(session_tags, []),  # for example ["string"]
                "onlyRunning": only_running,
                "showDisabled": show_disabled,
                "withShared": with_shared,
            },
        )

    def run_dtl(self, workspace_id, dtl_graph, agent_id=None):
        """run_dtl"""
        raise RuntimeError("Method is unavailable")

    def _run_plugin_task(
        self,
        task_type,
        agent_id,
        plugin_id,
        version,
        config,
        input_projects,
        input_models,
        result_name,
    ):
        """_run_plugin_task"""
        raise RuntimeError("Method is unavailable")

    def run_train(
        self,
        agent_id,
        input_project_id,
        input_model_id,
        result_nn_name,
        train_config=None,
    ):
        """run_train"""
        raise RuntimeError("Method is unavailable")

    def run_inference(
        self,
        agent_id,
        input_project_id,
        input_model_id,
        result_project_name,
        inference_config=None,
    ):
        """run_inference"""
        raise RuntimeError("Method is unavailable")

    def get_training_metrics(self, task_id):
        """get_training_metrics"""
        raise RuntimeError("Method is unavailable")

    def deploy_model(self, agent_id, model_id):
        """deploy_model"""
        raise RuntimeError("Method is unavailable")

    def get_import_files_list(self, id):
        """get_import_files_list"""
        raise RuntimeError("Method is unavailable")

    def download_import_file(self, id, file_path, save_path):
        """download_import_file"""
        raise RuntimeError("Method is unavailable")

    def create_task_detached(self, workspace_id, task_type: str = None):
        """create_task_detached"""
        raise RuntimeError("Method is unavailable")

    def upload_files(self, task_id, abs_paths, names, progress_cb=None):
        """upload_files"""
        raise RuntimeError("Method is unavailable")

    def initialize(self, task_id, template, data=None, state=None):
        """initialize"""
        d = take_with_default(data, {})
        if "notifyDialog" not in d:
            d["notifyDialog"] = None
        if "scrollIntoView" not in d:
            d["scrollIntoView"] = None

        s = take_with_default(state, {})
        fields = [
            {"field": TEMPLATE, "payload": template},
            {"field": DATA, "payload": d},
            {"field": STATE, "payload": s},
        ]
        resp = self._api.task.set_fields(task_id, fields)
        return resp

    def get_url(self, task_id):
        """get_url"""
        return f"/apps/sessions/{task_id}"

    def download_git_file(self, app_id, version, file_path, save_path):
        """download_git_file"""
        raise NotImplementedError()

    def download_git_archive(
        self,
        ecosystem_item_id,
        app_id,
        version,
        save_path,
        log_progress=True,
        ext_logger=None,
    ):
        """download_git_archive"""
        payload = {
            ApiField.ECOSYSTEM_ITEM_ID: ecosystem_item_id,
            ApiField.VERSION: version,
            "isArchive": True,
        }
        if app_id is not None:
            payload[ApiField.APP_ID] = app_id

        response = self._api.post("ecosystem.file.download", payload, stream=True)
        if log_progress:
            if ext_logger is None:
                ext_logger = logger

            length = -1
            # Content-Length
            if "Content-Length" in response.headers:
                length = int(response.headers["Content-Length"])
            progress = Progress("Downloading: ", length, ext_logger=ext_logger, is_size=True)

        mb1 = 1024 * 1024
        ensure_base_path(save_path)
        with open(save_path, "wb") as fd:
            log_size = 0
            for chunk in response.iter_content(chunk_size=mb1):
                fd.write(chunk)
                log_size += len(chunk)
                if log_progress and log_size > mb1:
                    progress.iters_done_report(log_size)
                    log_size = 0

    def get_info(self, module_id, version=None):
        """get_info"""
        data = {ApiField.ID: module_id}
        if version is not None:
            data[ApiField.VERSION] = version
        response = self._api.post("ecosystem.info", data)
        return response.json()

    def get_ecosystem_module_info(self, module_id, version=None) -> ModuleInfo:
        """get_module_info"""
        data = {ApiField.ID: module_id}
        if version is not None:
            data[ApiField.VERSION] = version
        response = self._api.post("ecosystem.info", data)
        return ModuleInfo.from_json(response.json())

    def get_ecosystem_module_id(self, slug: str):
        modules = self.get_list_all_pages(
            method="ecosystem.list",
            data={"filter": [{"field": "slug", "operator": "=", "value": slug}]},
            convert_json_info_cb=lambda x: x,
        )
        if len(modules) == 0:
            raise KeyError(f"Module {slug} not found in ecosystem")
        if len(modules) > 1:
            raise KeyError(
                f"Ecosystem is broken: there are {len(modules)} modules with the same slug: {slug}. Please, contact tech support"
            )
        return modules[0]["id"]

    # def get_sessions(self, workspace_id: int, filter_statuses: List[TaskApi.Status] = None):
    #     filters = [{"field": "type", "operator": "=", "value": "app"}]
    #     # filters = []
    #     if filter_statuses is not None:
    #         s = [str(status) for status in filter_statuses]
    #         filters.append({"field": "status", "operator": "in", "value": s})
    #     result = self._api.task.get_list(workspace_id=workspace_id, filters=filters)
    #     return result

    def get_sessions(
        self,
        team_id,
        module_id,
        # only_running=False,
        show_disabled=False,
        session_name=None,
        statuses: List[TaskApi.Status] = None,
    ) -> List[SessionInfo]:
        infos_json = self.get_list_all_pages(
            method="apps.list",
            data={
                "teamId": team_id,
                "filter": [{"field": "moduleId", "operator": "=", "value": module_id}],
                # "onlyRunning": only_running,
                "showDisabled": show_disabled,
            },
            convert_json_info_cb=lambda x: x,
            # validate_total=False,
        )
        if len(infos_json) == 0:
            # raise KeyError(f"App [module_id = {module_id}] not found in team {team_id}")
            return []
        if len(infos_json) > 1:
            raise KeyError(
                f"Apps list in team is broken: app [module_id = {module_id}] added to team {team_id} multiple times"
            )
        dev_tasks = []
        sessions = infos_json[0]["tasks"]

        str_statuses = []
        if statuses is not None:
            for s in statuses:
                str_statuses.append(str(s))

        for session in sessions:
            to_add = True
            if session_name is not None and session["meta"]["name"] != session_name:
                to_add = False
            if statuses is not None and session["status"] not in str_statuses:
                to_add = False
            if to_add is True:
                session["moduleId"] = module_id
                dev_tasks.append(SessionInfo.from_json(session))
        return dev_tasks

    def start(
        self,
        agent_id,
        app_id=None,
        workspace_id=None,
        description="",
        params=None,
        log_level="info",
        users_id=None,
        app_version=None,
        is_branch=False,
        task_name="run-from-python",
        restart_policy="never",
        proxy_keep_url=False,
        module_id=None,
        redirect_requests={},
    ) -> SessionInfo:
        users_ids = None
        if users_id is not None:
            users_ids = [users_id]

        new_params = {}
        if "state" not in params:
            new_params["state"] = params
        else:
            new_params = params

        if app_version is None:
            module_info = self.get_ecosystem_module_info(module_id)
            app_version = module_info.get_latest_release().get("version", "")

        result = self._api.task.start(
            agent_id=agent_id,
            app_id=app_id,
            workspace_id=workspace_id,
            description=description,
            params=new_params,
            log_level=log_level,
            users_ids=users_ids,
            app_version=app_version,
            is_branch=is_branch,
            task_name=task_name,
            restart_policy=restart_policy,
            proxy_keep_url=proxy_keep_url,
            module_id=module_id,
            redirect_requests=redirect_requests,
        )
        if len(result) != 1:
            raise ValueError(f"{len(result)} tasks started instead of one")
        return SessionInfo.from_json(result[0])

    def wait(
        self,
        id: int,
        target_status: TaskApi.Status,
        attempts: Optional[int] = None,
        attempt_delay_sec: Optional[int] = None,
    ):
        """wait"""
        return self._api.task.wait(
            id=id,
            target_status=target_status,
            wait_attempts=attempts,
            wait_attempt_timeout_sec=attempt_delay_sec,
        )

    def stop(self, id: int) -> TaskApi.Status:
        """stop"""
        return self._api.task.stop(id)

    def get_status(self, task_id: int) -> TaskApi.Status:
        return self._api.task.get_status(task_id)


# info about app in team
# {
#     "id": 7,
#     "createdBy": 1,
#     "moduleId": 16,
#     "disabled": false,
#     "userLogin": "admin",
#     "config": {
#         "icon": "https://user-images.githubusercontent.com/12828725/182186256-5ee663ad-25c7-4a62-9af1-fbfdca715b57.png",
#         "author": {"name": "Maxim Kolomeychenko"},
#         "poster": "https://user-images.githubusercontent.com/12828725/182181033-d0d1a690-8388-472e-8862-e0cacbd4f082.png",
#         "needGPU": false,
#         "headless": true,
#         "categories": ["development"],
#         "lastCommit": "96eca85e1fbed45d59db405b17c04f4d920c6c81",
#         "description": "Demonstrates how to turn your python script into Supervisely App",
#         "main_script": "src/main.py",
#         "sessionTags": [],
#         "taskLocation": "workspace_tasks",
#         "defaultBranch": "master",
#         "isPrivateRepo": false,
#         "restartPolicy": "never",
#         "slyModuleInfo": {"baseSlug": "supervisely-ecosystem/hello-world-app"},
#         "communityAgent": true,
#         "iconBackground": "#FFFFFF",
#         "integratedInto": ["standalone"],
#         "storeDataOnAgent": false,
#     },
#     "name": "Hello World!",
#     "slug": "supervisely-ecosystem/hello-world-app",
#     "moduleDisabled": false,
#     "provider": "sly_gitea",
#     "repositoryId": 42,
#     "pathPrefix": "",
#     "baseUrl": null,
#     "isShared": false,
#     "tasks": [
#         {
#             "id": 19107,
#             "type": "app",
#             "size": "0",
#             "status": "finished",
#             "startedAt": "2022-08-04T14:59:45.797Z",
#             "finishedAt": "2022-08-04T14:59:49.793Z",
#             "meta": {
#                 "app": {
#                     "id": 7,
#                     "name": "Hello World!",
#                     "version": "v1.0.4",
#                     "isBranch": false,
#                     "logLevel": "info",
#                 },
#                 "name": "",
#                 "params": {"state": {}},
#                 "hasLogs": true,
#                 "logsCnt": 60,
#                 "hasMetrics": false,
#                 "sessionToken": "PDVBF6ecX09FY75n7ufa8q_MTB28XI6XIMcJ1md4ogeN0FLTbIZyC91Js_9YkGpUQhQbCYyTE8Q=",
#                 "restartPolicy": "never",
#             },
#             "attempt": 1,
#             "archived": false,
#             "nodeId": 1,
#             "createdBy": 6,
#             "teamId": 7,
#             "description": "",
#             "isShared": false,
#             "user": "max",
#         }
#     ],
#     "repo": "https://github.com/supervisely-ecosystem/hello-world-app",
#     "repoKey": "supervisely-ecosystem/hello-world-app",
#     "githubModuleUrl": "https://github.com/supervisely-ecosystem/hello-world-app",
#     "repositoryModuleUrl": "https://github.com/supervisely-ecosystem/hello-world-app",
#     "teamId": 7,
# }


# infor about module in ecosystem



from supervisely.app import DataJson, StateJson
from supervisely.app.widgets import Widget


class BindedInputNumber(Widget):
    def __init__(
        self,
        width: int = 256,
        height: int = 256,
        min: int = 1,
        max: int = 10000,
        proportional: bool = False,
        widget_id: str = None,
    ):
        self._width = width
        self._height = height
        self._min = min
        self._max = max
        self._proportional = proportional
        self._disabled = False

        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        return {
            "options": {
                "proportions": {
                    "width": self._width,
                    "height": self._height
                }
            },
            "disabled": self._disabled
        }

    def get_json_state(self):
        return {
            "value": {
                "min": self._min,
                "max": self._max,
                "width": self._width,
                "height": self._height,
                "proportional": self._proportional
            }
        }

    @property
    def value(self):
        return self._width, self._height

    @value.setter
    def value(self, width, height):
        self._width = width
        self._height = height
        StateJson()[self.widget_id]["width"] = self._width
        StateJson()[self.widget_id]["height"] = self._height
        StateJson().send_changes()

    def get_value(self):
        width =  StateJson()[self.widget_id]['value']["width"]
        height =  StateJson()[self.widget_id]['value']["height"]
        return width, height

    @property
    def proportional(self):
        return self._proportional

    @property
    def min(self):
        return self._min

    @property
    def max(self):
        return self._max

    @min.setter
    def proportional(self, value):
        self._proportional = value
        DataJson()[self.widget_id]["proportional"] = self._proportional
        DataJson().send_changes()

    @min.setter
    def min(self, value):
        self._min = value
        DataJson()[self.widget_id]["min"] = self._min
        DataJson().send_changes()

    @max.setter
    def max(self, value):
        self._max = value
        DataJson()[self.widget_id]["max"] = self._max
        DataJson().send_changes()


    def disable(self):
        self._disabled = True
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()

    def enable(self):
        self._disabled = False
        DataJson()[self.widget_id]["disabled"] = self._disabled
        DataJson().send_changes()